package dk.nsp.idsas.util;

import org.apache.log4j.Logger;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Objects;
import java.util.UUID;

/**
 * <p>This class is adapted from: <a href="https://stackoverflow.com/a/40230410">https://stackoverflow.com/a/40230410</a></p>
 * <p>The namespace has been randomly generated using 'uuidgen' (<a href="https://linux.die.net/man/1/uuidgen">https://linux.die.net/man/1/uuidgen)</a></p>
 * <p>To compare output with 'uuidgen' (input: "test123"), use the following command:
 * <pre>$ uuidgen --sha1 --namespace 4cc1bbf5-ca68-4d89-a05f-1a51c4880b44 --name test123</pre>
 * </p>
 */
public class UUIDv5 {
    private static final Logger logger = Logger.getLogger(UUIDv5.class);

    public static final UUID NAMESPACE = UUID.fromString("4cc1bbf5-ca68-4d89-a05f-1a51c4880b44");

    public static UUID generateUUIDFromBytes(byte[] bytes) {
        MessageDigest md;

        try {
            md = MessageDigest.getInstance("SHA-1");
        } catch (NoSuchAlgorithmException e) {
            logger.error(e.getMessage(), e);
            throw new InternalError("SHA-1 not supported");
        }

        md.update(toBytes(Objects.requireNonNull(NAMESPACE, "namespace is null")));
        md.update(Objects.requireNonNull(bytes, "name is null"));

        byte[] sha1Bytes = md.digest();
        sha1Bytes[6] &= 0x0f;  /* clear version        */
        sha1Bytes[6] |= 0x50;  /* set to version 5     */
        sha1Bytes[8] &= 0x3f;  /* clear variant        */
        sha1Bytes[8] |= 0x80;  /* set to IETF variant  */

        return fromBytes(sha1Bytes);
    }

    private static UUID fromBytes(byte[] data) {
        // Based on the private UUID(bytes[]) constructor
        long msb = 0;
        long lsb = 0;
        assert data.length >= 16;
        for (int i = 0; i < 8; i++)
            msb = (msb << 8) | (data[i] & 0xff);
        for (int i = 8; i < 16; i++)
            lsb = (lsb << 8) | (data[i] & 0xff);
        return new UUID(msb, lsb);
    }

    private static byte[] toBytes(UUID uuid) {
        // inverted logic of fromBytes()
        byte[] out = new byte[16];
        long msb = uuid.getMostSignificantBits();
        long lsb = uuid.getLeastSignificantBits();
        for (int i = 0; i < 8; i++)
            out[i] = (byte) ((msb >> ((7 - i) * 8)) & 0xff);
        for (int i = 8; i < 16; i++)
            out[i] = (byte) ((lsb >> ((15 - i) * 8)) & 0xff);
        return out;
    }
}