package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClient;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClientException;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.HashingUtil;
import dk.nsp.idsas.util.WhitelistType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.PatientIdClassificationType;
import org.apache.log4j.Logger;
import dk.nsp.idsas.util.FaultCode;

import java.time.ZonedDateTime;

public class BlurringService {
    private static final Logger logger = Logger.getLogger(BlurringService.class);

    private final WhitelistDao whitelistDao;
    private final BlurringDao blurringDao;
    private final AuditLogger auditLogger;
    private final PersonInformationClient personInformationClient;
    private final byte[] patientIdSalt;
    private final int expiryMaxYears;

    public BlurringService(WhitelistDao whitelistDao, BlurringDao blurringDao, byte[] patientIdSalt,
                           AuditLogger auditLogger, PersonInformationClient personInformationClient,
                           int expiryMaxYears) {
        this.whitelistDao = whitelistDao;
        this.blurringDao = blurringDao;
        this.auditLogger = auditLogger;
        this.personInformationClient = personInformationClient;
        this.patientIdSalt = patientIdSalt;
        this.expiryMaxYears = expiryMaxYears;
    }

    public void createBlurring(String organizationId, String organizationIdType, String patientId, String patientIdType, ZonedDateTime endDateTime) throws IdsasServiceException {

        // Tjek at organizationId er whitelisted til at kalde createBlurring
        boolean isWhiteListed = whitelistDao.isWhitelisted(organizationId, WhitelistType.BLURRING);
        if (!isWhiteListed) {
            throw new IdsasServiceException(ErrorMessages.cvrWhitelistError(organizationId, WhitelistType.BLURRING), FaultCode.CREATE_BLURRING_ERROR, true);
        }

        // Tjek at patientId er gyldig (via "personinformation")
        if (patientIdType.equals(PatientIdClassificationType.CPR.value())) {
            boolean patientIdIsValid = false;
            try {
                patientIdIsValid = personInformationClient.personExists(patientId);
                if (!patientIdIsValid) {
                    throw new IdsasServiceException(ErrorMessages.cprIsNotValidError(patientId), FaultCode.CREATE_BLURRING_ERROR, true);
                }
            } catch (PersonInformationClientException e) {
                logger.error(e.getMessage(), e);
                throw new IdsasServiceException(e.getMessage(), FaultCode.CREATE_BLURRING_ERROR, false);
            }
        }

        // Tjek at sløringen ikke ligger mere end 'expiryMaxYears' år fremme i tiden
        if (endDateTime.isAfter(ZonedDateTime.now().plusYears(expiryMaxYears))) {
            throw new IdsasServiceException(ErrorMessages.maxExpiryDateExceeded(), FaultCode.CREATE_BLURRING_ERROR, true);
        }

        // PatientId hashes!
        String hashedPatientId = HashingUtil.createPatientIdHash(patientId, patientIdSalt);

        // Entity oprettes og gemmes
        BlurringEntity entity = new BlurringEntity(
                hashedPatientId,
                patientIdType,
                organizationId,
                organizationIdType,
                endDateTime.toInstant());

        blurringDao.insert(entity);

        // Audit logning
        auditLogger.logCreateBlurring(patientId, patientIdType, hashedPatientId, organizationId, organizationIdType, entity.getExpiryDate());
    }

}
