package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.WhitelistType;

import java.security.SecureRandom;

public class SaltService {
    private final SaltDao saltDao;
    private final AuditLogger auditLogger;
    private final WhitelistDao whitelistDao;
    private final SecureRandom secureRandom;

    public SaltService(SaltDao saltDao, WhitelistDao whitelistDao) {
        this(saltDao, whitelistDao, null, new SecureRandom());
    }

    public SaltService(SaltDao saltDao, WhitelistDao whitelistDao, AuditLogger auditLogger) {
        this(saltDao, whitelistDao, auditLogger, new SecureRandom());
    }

    public SaltService(SaltDao saltDao, WhitelistDao whitelistDao,
                       AuditLogger auditLogger, SecureRandom secureRandom) {
        this.saltDao = saltDao;
        this.auditLogger = auditLogger;
        this.whitelistDao = whitelistDao;
        this.secureRandom = secureRandom;
    }

    public byte[] getCurrentSalt(IdsasActor actor) throws IdsasServiceException {
        validateSystemUser(actor.getActorType());
        validateClientWhitelisted(actor.getClientKey());
        byte[] salt = saltDao.getLatestSalt();
        auditLogger.logGetCurrentSalt(actor.getClientKey(), "cvr", salt);
        return salt;
    }

    public void renewSalt() {
        byte[] salt = new byte[16];
        secureRandom.nextBytes(salt);
        saltDao.insert(salt);
    }

    private void validateSystemUser(IdsasActor.IdsasActorType actorType) throws IdsasServiceException {
        if (!actorType.equals(IdsasActor.IdsasActorType.SYSTEM)) {
            throw new IdsasServiceException(ErrorMessages.userTypeNotSupported(actorType.name()), FaultCode.GET_SALT_ERROR, true);
        }
    }

    private void validateClientWhitelisted(String clientKey) throws IdsasServiceException {
        if (!whitelistDao.isWhitelisted(clientKey, WhitelistType.SALT)) {
            throw new IdsasServiceException(ErrorMessages.clientKeyWhitelistError(clientKey, WhitelistType.SALT), FaultCode.GET_SALT_ERROR, true);
        }
    }
}
